import QtQuick 1.1
import Common 1.0

// FIXME: crash if apply object is reparented to belt pack and screen changed

// Object that can be applied to the other objects.
// Any Obj based component can accept ApplyObj applying.
Obj {

    // Indicates whether the object should be moved to the belt pack on the first click.
    // If false the object should be applied directly to other object.
    property bool beltPackCollectable: false
    // when state = "ReadyToApply", parent change to game
    property bool forceParentToGame
    // Dictates how much times the object can be applied. If applyTimes == 0 there will be infinity applying
    property int applyTimes: 1
    // Apply obj name - often used for tooltip
    property string name
    // The property can be utilized by simple apply objects
    property url imageSource
    // Source of image to be used as icon for debug pupose (ApplyObj-s list)
    property url debugImageSource: imageSource
    // If true and applyTimes > 0 state is not changed on apply
    property bool keepStateOnApply

    cursor: "Hand"
    hintable: beltPackCollectable


    // Puts the object to the belt pack if it's possible
    function putToBeltPack() {
        if (beltPackCollectable && state == "") {
            state = "InBeltPack"
            beltPack.itemAdded(self)
            // ApplyObj from another screen or location can be put to the belt pack, so, load its images
            Utils.setInactive(self, false)
        }
    }


    id: self

    // Point where mouse was pressed on the object. Not filled by the user.
    property int mouseOffsetX
    property int mouseOffsetY
    // Point where mouse was pressed on the game component. Not filled by the user.
    property int mouseInitialX
    property int mouseInitialY
    // The index below is optionally filled by the belt pack
    property int beltPackIndex
    // Internal property to correctly load game state
    property int _beltPackItemIndex
    // Internal property to indicate that mouse is pressed
    property bool _isMousePressedInBeltPack


    onStateChanged: {
        game.applyObjStateChanged(self)
    }

    onPressed: {
        if (!draggingApplyObj) {
            _isMousePressedInBeltPack = state.search(/InBeltPack\w*/) !== -1
            if ((state === "" && !beltPackCollectable) || _isMousePressedInBeltPack) {
                mouseOffsetX = mouse.x
                mouseOffsetY = mouse.y
                var point = mapToItem(game, mouse.x, mouse.y)
                mouseInitialX = point.x
                mouseInitialY = point.y
                if (!beltPackCollectable) {
                    state = "ReadyToApply"
                }
            }
        }
    }

    onPositionChanged: {
        if (mouse.buttons === Qt.LeftButton && state.search(/InBeltPack\w*/) !== -1) {
            var dy = Math.abs(mouse.y - mouseOffsetY)
            var dx = Math.abs(mouse.x - mouseOffsetX)
            if ((beltPack.horizontalFlickable && dy > dx && dy > 10)
                    || (beltPack.verticalFlickable && dx > dy && dx > 10)) {
                state = "ReadyToApply"
            }
        }
    }

    onClicked: {
        if (state.search(/InBeltPack\w*/) !== -1) {
            state = "ReadyToApply"
        } else {
            putToBeltPack()
        }
    }

    Connections {
        target: self.state === "ReadyToApply" ? game : null
        // This slot handles released signal on the game component only
        onReleased: {
            _isMousePressedInBeltPack = false
            // Interpret small drag as a click and do not change the state
            if (Math.abs(mouse.x - mouseInitialX) + Math.abs(mouse.y - mouseInitialY) > 15) {
                // Detect applying target and try to apply
                var c = Utils.objAt(game, dragX, dragY, self)
                if (c) {
                    if (c.acceptApplyObj !== undefined && c.acceptApplyObj(self)) {
                        if (--applyTimes === 0) {
                            state = "Applied"
                            return
                        }
                        if (keepStateOnApply) {
                            return
                        }
                    } else if (c.keepUnacceptedApplyObjDragged) {
                        return
                    }
                }
                returnAfterDragAndFireSignal()
            }
        }
        // Work around a programmatic screen switch
        onScreenLoaded: returnAfterDragAndFireSignal()
        onLocationChanged: returnAfterDragAndFireSignal()
    }

    Connections {
        target: _isMousePressedInBeltPack ? beltPack : null
        onDragStopped: {
            state = "ReadyToApply"
        }
        onMovementEnded: {
            _isMousePressedInBeltPack = false
        }
    }

    function returnAfterDrag() {
        state = beltPackCollectable ? "InBeltPack" : ""
    }

    function returnAfterDragAndFireSignal() {
        var itemX = x, itemY = y
        returnAfterDrag()
        if (beltPackCollectable) {
            beltPack.itemReturned(self, itemX, itemY)
        }
    }

    // Independent state group allows derived components to utilize build-in state properties.
    StateGroup {
        state: self.state
        states: [
            State { // This state is here as a work around of the problem of placing this state in InventoryObj
                name: "InBeltPackSharedSlot"; extend: "InBeltPack"
                PropertyChanges { target: self; parent: predecessor }
            },
            State {
                name: "InBeltPack"
                PropertyChanges {
                    target: self
                    hoverEnabled: true
                    x: 0; y: 0
                    parent: beltPack.container
                    visible: true // There is a lot of cases when the obj is hidden by default
                }
            },
            State {
                name: "ReadyToApply"
                PropertyChanges {
                    target: self
                    // Make it the top child of the game or screen
                    parent: forceParentToGame || beltPackCollectable || popupLayer.isWindowOpen ? game : screen
                    x: dragX - width * 0.5 // Bind x, y for dragging
                    y: dragY - height * 0.5
                    z: 100 
                    visible: true // There is a lot of cases when the obj is hidden by default
                }
            },
            State {
                name: "Applied"
                PropertyChanges { target: self; visible: false; opacity: 0 }
                StateChangeScript { script: {
                        if (beltPackCollectable) {
                            beltPack.itemRemoved(self); Utils.setInactive(self, true)
                        }
                    }
                }
            }
        ]
    }


    // Save/load support
    function saveApplyObjState() {
        var data = state ? {state : state} : {}

        // Work around for saving when the object is dragged
        if (state === "ReadyToApply") {
            data.state = beltPackCollectable ? "InBeltPack" : ""
        }

        if (applyTimes !== 1) {
            data.applyTimes = applyTimes
        }
        if (_beltPackItemIndex) {
            data.beltPackItemIndex = _beltPackItemIndex
        }
        return data
    }
    function loadApplyObjState(data) {
        applyTimes = data.applyTimes === undefined ? 1 : data.applyTimes
        _beltPackItemIndex = data.beltPackItemIndex || 0
        if (data.state) {
            if (data.state.search(/InBeltPack\w*/) !== -1) {
                self.state = ""
                beltPack.loadedObjects.setAt(_beltPackItemIndex, self)
            } else {
                self.state = data.state
            }
        } else {
            self.state = ""
        }
    }
}
