import QtQuick 1.1
import "EditPoints"

/// Adds debug facilities to the application
Item {
    // Debug children must be DebugMenuButtons
    default property alias buttons: debugMenu.buttons

    property alias debugMenu: debugMenu

    property string helpText: '
    <h3>This is a placeholder for the help text.</h3>
    You can use <i>StyledText</i> to describe your hot keys and cheats.<br/>
    StyledText is an optimized format supporting some basic text styling markup, in the style of html 3.2:
    <ul>
        <li><font size="4" color="#ff0000">font size and color</font></li>
        <li><b>bold</b></li>
        <li><i>italic</i></li>
    </ul>
    <p>
    Modify Debug.helpText property in Game.qml to set up the text.
    </p>
    '

    property alias fps: fps
    property alias itemCounter: itemCounter
    property alias layering: layering
    property alias memoryInfo: memoryInfo
    property alias snapshots: snapshots    
    property alias animationTime: animationTime

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading
    
    id: self
    x: visibleArea.x; y: visibleArea.y
    width: visibleArea.width; height: visibleArea.height
    z: 15000
    visible: developmentConfig.get("debugEnabled") === "true"

    DebugMenu {
        id: debugMenu
        z: 100

        DebugMenuButton {
            text: "Help (F1)"; hotKey: Qt.Key_F1
            target: Help {}
        }
        DebugMenuButton {
            text: "Obj highlights (H)"; hotKey: Qt.Key_H
            target: ObjHighlightsAndInfo {}
        }
        DebugMenuButton {
            text: "Snapshots (F3)"; hotKey: Qt.Key_F3
            target: snapshots
        }
        DebugMenuButton {
            text: "Edit points (F4)"; hotKey: Qt.Key_F4
            target: EditPointsDialog {}
        }
        DebugMenuButton {
            text: "Navigation (L)"; hotKey: Qt.Key_L
            target: Navigation {}
        }
        DebugMenuButton {
            text: "Apply objects (B)"; hotKey: Qt.Key_B
            target: ApplyObjList {}
        }
        DebugMenuButton {
            text: "Close (ESC)"; hotKey: Qt.Key_Escape
        }
        DebugMenuButton {
            text: "Fill hint (U)"; onClicked: self.fillHint(game)
            hotKey: Qt.Key_U
        }
        DebugMenuButton {
            text: "Prev screen (Z)"; hotKey: Qt.Key_Z
            onClicked: switchPrevScreen()
        }
        DebugMenuButton {
            text: "Next screen (X)"; hotKey: Qt.Key_X
            onClicked: switchNextScreen()
        }
        DebugMenuButton {
            text: "Audio info (A)"; hotKey: Qt.Key_A
            target: AudioInfo {}
        }
        DebugMenuButton {
            text: "Reload (F5)"; hotKey: Qt.Key_F5
            onClicked: fullReload()
        }
        DebugMenuButton {
            text: "Slow anim. (F8)"; hotKey: Qt.Key_F8
            onClicked: slowAnimation()
        }
        DebugMenuButton {
            text: "Step anim. (F9)"; hotKey: Qt.Key_F9
            onClicked: stepAnimation()
        }

        Component.onCompleted: {
            // Scan children for DebugMenuButton-s
            for (var i = 0; i < buttons.length; ++i) {
                var button = buttons[i]
                if (button.hotKey !== undefined && button.target) {
                    button.target.visible = false
                    button.target.parent = self
                }
            }

            // Focus self to work around the focus problem after reload
            self.forceActiveFocus()
            debugMenu.forceActiveFocus()
        }
    }

    Item {
        id: debugInfo
        anchors.fill: parent
        visible: developmentConfig.get("showDebugInfo") === "true"

        Text {
            id: animationTime
            anchors.right: parent.right
            color: "green"
            text: (fineAnimationDriver.updateTimeMs * 100 / fineAnimationDriver.totalTimeCounter).toFixed(1) + "%"
            font.pixelSize: 14
        }

        Text {
            id: fps
            anchors.right: animationTime.left
            anchors.rightMargin: 10
            color: "Red"
            text: root.fps
            font.pixelSize: 14
        }

        Layering {
            id: layering
            anchors.rightMargin: 10; anchors.right: fps.left
        }

        MemoryInfo {
            id: memoryInfo
            anchors.rightMargin: 10; anchors.right: layering.left
        }

        Text {
            id: itemCounter
            anchors.rightMargin: 10; anchors.right: memoryInfo.left
            color: "Grey"
            Timer {
                repeat: true; running: true
                onTriggered: itemCounter.text = Utils.countVisibleItems(root) + "(" + Utils.countItems(root)
                             + "," + Utils.countObjects(root) + ") items"
            }
        }
    }

    // Allow access to Snapshots' function. So declared here.
    Snapshots {
        id: snapshots
    }

    // Closes all debug windows except 'skip' one and menu, fps
    function closeAll(skip) {
        for (var i = 0; i < children.length; ++i) {
            var child = children[i]
            if (child !== skip && child !== debugMenu && child !== debugInfo) {
                child.visible = false
            }
        }
    }

    // The small cheat fills all visible hint timers
    function fillHint(item) {
        if (item.visible && item.opacity && item.scale) {
            if (item.fillingTime) {
                item.fill()
            }
            for (var i = 0; i < item.children.length; ++i) {
                fillHint(item.children[i])
            }
        }
    }

    function switchNextScreen() {
        if (activeLocation) {
            var index = activeLocation.activeScreenIndex + 1
            if (index >= activeLocation.screens.length) {
                index = 0
            }
            activeLocation.switchToScreen(activeLocation.screens[index])
        }
    }

    function switchPrevScreen() {
        if (activeLocation) {
            var index = activeLocation.activeScreenIndex - 1
            if (index < 0) {
                index = activeLocation.screens.length - 1
            }
            activeLocation.switchToScreen(activeLocation.screens[index])
        }
    }

    function fullReload() {
        // TODO: reload thru the player
        if (audio) {
            if (audio.currentMusic) audio.currentMusic.stop()
            if (audio.awardMusicSource) audio.awardMusicSource.stop()
        }
        Utils.clearQmlCache(root)
        Utils.setInactive(game, true)
        var source = gameLoader.source
        gameLoader.source = ""
        Utils.clearPixmapCache()
        gameLoader.source = source
    }

    function reloadScreen() {
        if (activeLocation && activeLocation.activeScreen) {
            Utils.clearQmlCacheItem(activeLocation.activeScreen)
            activeLocation.switchToScreen(null)
            _delayReloadingTimer.start()
        }
    }

    Timer {
        id: _delayReloadingTimer
        interval: 50
        onTriggered: {
            Utils.clearPixmapCache()
            activeLocation.switchToScreen(activeLocation.screens[activeLocation.activeScreenIndex])
        }
    }

    function slowAnimation() {
        fineAnimationDriver.timeFactor = fineAnimationDriver.timeFactor === 1 ? 5 : 1
    }

    function stepAnimation() {
        fineAnimationDriver.timeFactor = 0
        fineAnimationDriver.advanceForDebug()
    }
}
