import QtQuick 1.1
import Common 1.0

// Dialog for point list editing
PointListEditor {

    id: self
    acceptedButtons: Qt.LeftButton | Qt.RightButton


    function selectedArea() {
        if (polygon.state === "Selected") {
            return displayPolygong
        } else if (spline.state === "Selected") {
            return displaySpline
        } else if (area.state === "Selected") {
            return displayArea
        }
    }

    onPointsCommited: {
        if (pointsCode.activeFocus) offset.forceActiveFocus()
        pointsCode.text = selectedArea().convertPointsToString(offset.text)
    }

    onPressed: {
        if (mouse.button === Qt.RightButton) {
            var obj = Utils.objAt(game, mouse.x, mouse.y, self)
            if (obj) {
                console.log("Take offset of", obj)
                var p = obj.mapToItem(game, 0, 0)
                offset.text = p.x + ", " + p.y
            }
        }
    }


    Rectangle {
        id: dialog
        border.width: 2; border.color: "Blue"
        radius: 8; width: 380; height: 200
        x: (parent.width - width) / 2; y: parent.height - 210

        Rectangle {
            id: header
            anchors.left: parent.left; anchors.right: parent.right
            anchors.top: parent.top; anchors.margins: 3
            height: 5; radius: 2; color: "LightSteelBlue"

            MouseArea {
                id: mouseArea
                anchors.fill: parent; hoverEnabled: true; drag.target: dialog
                states: State {
                    when: mouseArea.containsMouse
                    PropertyChanges { target: header; color: "Red" }
                }
            }
        }

        // Controls
        QtQuickItem {
            anchors.left: parent.left; anchors.right: parent.right
            anchors.top: header.bottom; anchors.bottom: parent.bottom
            anchors.margins: 3

            // Offset label and offset text input
            Text { id: offsetLabel; text: "Offset: "; x: 10; y: 20 }
            Rectangle {
                anchors.left: offsetLabel.right
                anchors.top: offsetLabel.top; anchors.topMargin: -4
                width: 90; height: 20; border.width: 1; clip: true

                TextInput { id: offset; text: "0, 0"
                    anchors.fill: parent; anchors.topMargin: 3; anchors.leftMargin: 4
                    onTextChanged: pointsCode.text = selectedArea().convertPointsToString(text)
                }
            }

            // Toggle polygong
            EPToggleButton {
                id: polygon; width: 70; x: 175; y: 18;
                state: "Selected"; text: "Polygon"
                onClicked: { polygon.state = "Selected"; spline.state = ""; area.state = ""; pointsCommited(points) }
            }
            // Toggle sprite
            EPToggleButton {
                id: spline; width: 60; text: "Spline"
                anchors.top: polygon.top; anchors.left: polygon.right
                onClicked: { spline.state = "Selected"; polygon.state = ""; area.state = ""; pointsCommited(points) }
            }
            EPToggleButton {
                id: area; width: 60; text: "Area"
                anchors.top: spline.top; anchors.left: spline.right
                onClicked: { spline.state = ""; polygon.state = ""; area.state = "Selected"; pointsCommited(points) }
            }

            // Code label and code text edit
            Text { id: pointsCodeLabel; text: "Code:"; anchors.left: offsetLabel.left;
                anchors.top: offsetLabel.bottom; anchors.topMargin: 20 }
            Rectangle {
                anchors.left: offsetLabel.left
                anchors.right: parent.right; anchors.rightMargin: 10
                anchors.top: pointsCodeLabel.bottom; anchors.topMargin: 6
                anchors.bottom: parent.bottom; anchors.bottomMargin: 10
                border.width: 1; clip: true

                Flickable {
                    anchors.fill: parent; anchors.margins: 4
                    contentWidth: pointsCode.width; contentHeight: pointsCode.height
                    TextEdit {
                        id: pointsCode; wrapMode: TextEdit.Wrap; width: 270
                        onTextChanged: if (pointsCode.activeFocus) selectedArea().convertCodeToPoints(text, offset.text)
                    }
                }
            }

            // Clear button
            EPButton {
                id: clear; text: "Clear"; anchors.top: pointsCodeLabel.top
                anchors.right: parent.right; anchors.rightMargin: 10
                onClicked: {
                    pointsCode.text = ""
                    selectedArea().convertCodeToPoints(pointsCode.text, "0,0")
                }
            }
            // Paste button
            EPButton {
                id: paste; text: "Paste"; anchors.top: pointsCodeLabel.top
                anchors.right: clear.left; anchors.rightMargin: 10
                onClicked: {
                    pointsCode.paste()
                    selectedArea().convertCodeToPoints(pointsCode.text, offset.text)
                }
            }
            // Copy button
            EPButton {
                id: copy; text: "Copy"; anchors.top: pointsCodeLabel.top
                anchors.right: paste.left; anchors.rightMargin: 10
                onClicked: { pointsCode.selectAll(); pointsCode.copy() }
            }
        }

        // Help
        Rectangle {
            id: help
            anchors.margins: 5; anchors.topMargin: -2
            opacity: 0
            anchors.left: parent.left; anchors.right: parent.right
            anchors.top: question.bottom; anchors.bottom: parent.bottom

            Text {
                anchors.fill: parent; anchors.margins: 18
                text: "Click to add a point\n"
                      + "Drag the point to change its position\n"
                      + "Right click on the point to delete it\n"
                      + "Right click on the object to fill\n the offset field"
            }
        }
        Text {
            id: question; text: "?"; font.underline: true; color: "Blue"
            anchors.right: parent.right; anchors.top: header.top; anchors.margins: 5
            MouseArea {
                id: questionArea; anchors.fill: parent; hoverEnabled: true
                states: State {
                    when: questionArea.containsMouse
                    PropertyChanges { target: help; opacity: 1 }
                }
                transitions: Transition {
                    PropertyAnimation { target: help; properties: "opacity"; duration: 300 }
                }
            }
        }
    }

    // Points display components
    DisplayArea {
        id: displayArea; z: -1
        visible: area.state === "Selected"
        offset: offset.text
        pointsCode: pointsCode.text
    }
    // Points display components
    DisplayPolygon {
        id: displayPolygong; z: -1
        visible: polygon.state === "Selected"
        offset: offset.text
        pointsCode: pointsCode.text
    }
    DisplaySpline {
        id: displaySpline; z: -1
        visible: spline.state === "Selected"
        offset: offset.text
        pointsCode: pointsCode.text
    }
}
