import QtQuick 1.1
import Common 1.0


// UI to manage debug snapshots
QtQuickItem {

    // Readonly
    property string folder: Persistence.writableLocation + "/_Snapshots_/"
    property alias list: list

    id: self
    width: 600
    height: 500
    anchors.centerIn: parent

    onVisibleChanged: if (visible) {
        Persistence.mkpath(folder) // Create the dir if empty
        list.refresh()
    }

    // Saves a snapshot. If name omitted the current date and time is used as a name.
    function saveSnapshot(name) {
        if (name === undefined) {
            name = Qt.formatDateTime(new Date(), "yy_MM_dd-hh_mm_ss")
        }
        var path = self.folder + name
        console.log("Save snapshot", path)
        persistenceController.forceSave()
        Persistence.copyDir(persistenceController.profileManager.profilesDir + "/" + persistenceController.profileName, path)
        list.refresh()
        return name
    }

    // Loads a snapshot.
    function loadSnapshot(name) {
        var path = self.folder + name
        console.log("Load snapshot", path)
        if (activeLocation) {
            activeLocation.switchToScreen(null)
        }
        Persistence.copyDir(path, persistenceController.profileManager.profilesDir + "/" + persistenceController.profileName)
        persistenceController.loadGame()
        switchToLocation(locations.at(activeLocationIndex))
    }

    // Reset cursor with ObjBase on this area
    ObjBase {
        anchors.fill: parent
    }

    // Header
    Rectangle {
        anchors.left: parent.left
        anchors.right: parent.right
        height: 34
        color: "DarkBlue"
        y: -32
        radius: 4
        Text {
            text: "Game state snapshots"
            color: "White"
            anchors.centerIn: parent
            font.pixelSize: 17
            font.weight: Font.Bold
        }
    }

    // Left panel with list box
    Rectangle {
        id: leftPanel
        radius: 4
        border.width: 2
        width: 306
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.bottom: parent.bottom

        ListView {
            id: list
            anchors.fill: parent
            anchors.margins: 3
            spacing: 2
            clip: true
            focus: true
            highlight: Rectangle { color: "LightSteelBlue" }

            delegate: Text {
                width: 300
                height: 24
                x: 10
                font.pixelSize: 16
                text: fileName
            }

            MouseArea {
                anchors.fill: parent
                preventStealing: true
                onClicked: {
                    var i = list.indexAt(mouseX, mouseY + list.contentY)
                    if (i >= 0) {
                        list.currentIndex = i
                    }
                }
                onDoubleClicked: {
                    var i = list.indexAt(mouseX, mouseY + list.contentY)
                    if (i >= 0) {
                        list.currentIndex = i
                        loadButton.clicked()
                    }
                }
            }

            function refresh() {
                // Lazy model creation to speed up loading
                if (!model) {
                    model = Qt.createQmlObject("import Qt.labs.folderlistmodel 1.0; "
                          + "FolderListModel { sortField: FolderListModel.Name; folder: 'file:///' + self.folder; sortReversed: true }",
                                               list, "Snapshots.qml")
                    model.refresh()
                } else {
                    model.nameFilters = model.nameFilters ? "*.*" : "*"
                    var m = model; list.model = null; list.model = m
                    model.refresh()
                }
            }
        }
    }

    // Right panel
    Rectangle {
        radius: 4
        border.width: 2
        color: "#f2f2f2"
        anchors.left: leftPanel.right
        anchors.right: parent.right
        anchors.top: parent.top
        anchors.bottom: parent.bottom

        QtQuickItem {
            x: 30
            y: 40

            DebugButton {
                id: saveButton
                text: "Save snapshot"
                onClicked: {
                    var txt = textInput.text.trim()
                    if (txt !== "") {
                        saveSnapshot(txt)
                        debug.closeAll()
                    }
                }
            }
            Rectangle {
                id: textInputRect
                height: 20
                width: 220
                border.width: 1
                anchors.top: saveButton.bottom
                anchors.topMargin: 10
                TextInput {
                    id: textInput
                    anchors.fill: parent
                    anchors.topMargin: 3
                    anchors.leftMargin: 2
                    anchors.rightMargin: 2
                    font.pixelSize: 16
                    text: self.visible ? Qt.formatDateTime(new Date(), "yy_MM_dd-hh_mm_ss") : "???"
                }
            }

            DebugButton {
                id: deleteButton
                anchors.top: textInputRect.bottom
                anchors.topMargin: 40
                text: "Delete selected snapshot"
                selectedColor: "Red"
                onClicked: {
                    if (list.currentItem) {
                        var path = self.folder + list.currentItem.text
                        console.log("Delete snapshot", path)
                        Persistence.removeDir(path)
                        list.refresh()
                    }
                }
            }
            DebugButton {
                id: loadButton
                anchors.top: deleteButton.bottom
                anchors.topMargin: 20
                text: "Load selected snapshot"
                selectedColor: "Blue"
                onClicked: {
                    if (list.currentItem) {
                        loadSnapshot(list.currentItem.text)
                        debug.closeAll()
                    }
                }
            }
            DebugButton {
                anchors.top: loadButton.bottom
                anchors.topMargin: 230
                text: "Close"
                selectedColor: "Brown"
                onClicked: debug.closeAll()
            }
        }
    }
}
