import QtQuick 1.1
import Common 1.0

DiaryCover {
    property bool closed: true
    property real closedScale: 0.15
    property real closedCx: 150
    property real closedCy: 550

    property url pageLeftSource: "Images/PageLeft.png"
    property url pageRightSource: "Images/PageRight.png"

    property int currentSheetIndex
    property int sheetsCount: Math.ceil(pageSources.count() / 2)
    property ObjectContainer pageSources

    property int minTurnSheetIndex
    property int maxTurnSheetIndex: sheetsCount

    property alias sheetContainer: sheetContainer // Able to adjust margin
    property alias deadPageContentParent: deadPageContentParent // To be used for the page source management

    property variant pageDirt

    // Content for pages with index >= pageSources.count()
    property QtQuickItem overPageSourceFront
    property QtQuickItem overPageSourceBack

    // Read-only
    property bool pageDragging

    id: diary
    x: closed ? closedCx - width * 0.5 : (parent.width - width) * 0.5
    y: closed ? closedCy - height * 0.5 : (parent.height - height) * 0.5
    scale: closed ? closedScale : 1
    closeAngle: closed ? 88 : 0

    // Binding can be destroyed, so, use assignment
    onPageSourcesChanged: sheetsCount = Math.ceil(pageSources.count() / 2)

    function turnSheets(sheetIndex) {
        if (!sheetTurningAnimation.running) {
            sheetTurningAnimation.targetSheetIndex = sheetIndex
            if (Math.abs(sheetIndex - currentSheetIndex) > 9) {
                // Skip all pages in range (currentSheetIndex + 4, sheetIndex - 4)
                if (sheetIndex > currentSheetIndex) {
                    sheetContainer.omittedSheetsBegin = currentSheetIndex + 4
                    sheetContainer.omittedSheetsEnd = sheetIndex - 4
                } else {
                    sheetContainer.omittedSheetsEnd = currentSheetIndex - 4
                    sheetContainer.omittedSheetsBegin = sheetIndex + 4
                }
            }
            sheetTurningAnimation.tick()
        }
    }

    onClosedChanged: {
        // Debug check. You can get this error message if you close diary while sheets turning
        if (sheetContainer.children.length > 4) {
            console.log("ERROR: too much sheets in diary")
        }

        // Fire signals in game
        if (closed) {
            diaryClosed(diary)
        } else {
            diaryOpened(diary)
        }
    }

    Connections {
        target: pageSources
        onDataChanged: sheetsCount = Math.ceil(pageSources.count() / 2)
    }

    Behavior on x { NumberAnimation { duration: 300 } }
    Behavior on y { NumberAnimation { duration: 300 } }
    Behavior on scale { NumberAnimation { duration: 300 } }
    Behavior on closeAngle { NumberAnimation { duration: 300 } }

    MouseArea {
        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

        id: closeClick
        anchors.fill: closeButton
        hoverEnabled: true
        onClicked: closed = true
        z: 2

        Binding {
            target: closeButton; property: "transform"; value: Translate { 
                y: 10 * closeClick.containsMouse
                Behavior on y {
                    NumberAnimation { duration: 400; easing.type: Easing.InOutQuint }
                }
            }
        }
    }

    SheetContainer {
        id: sheetContainer
        anchors.topMargin: -18
        z: 2
    }

    // Multiple page turning helper
    PauseAnimation {
        property int targetSheetIndex
        property int idleTicks
        id: sheetTurningAnimation
        duration: 150
        onCompleted: tick()
        function tick() {
            if (currentSheetIndex < targetSheetIndex) {
                currentSheetIndex = sheetContainer.nextSheetIndex(currentSheetIndex)
                idleTicks = 3
                start()
            } else if (currentSheetIndex > targetSheetIndex) {
                currentSheetIndex = sheetContainer.prevSheetIndex(currentSheetIndex)
                idleTicks = 3
                start()
            } else if (--idleTicks <= 0) {
                sheetContainer.omittedSheetsBegin = sheetContainer.omittedSheetsEnd = 0
            } else {
                start()
            }
        }
    }

    // Special item for dead page content
    QtQuickItem {
        id: deadPageContentParent; visible: false

        property bool isTransient: true // Do not save/load state
        property bool isSkipForEnumeration: true // Do not enumerate on saving/loading
    }



//    function numVisibleComponents(component) {
//        if (!component.visible) {
//            return 0
//        }

//        var v = component.source !== undefined
//        if (component.pageIndex !== undefined) console.debug("Page visible:", component.pageIndex)
//        for (var i = 0; i < component.children.length; ++i) {
//            v += numVisibleComponents(component.children[i])
//        }
//        return v
//    }

//    onScaleChanged: {
//        if (scale > 0.8) {
//            console.debug("Diary visible elements", numVisibleComponents(diary))
//        }
//    }

//    onCurrentSheetIndexChanged: {
//        if (currentSheetIndex > 5) {
//            console.debug("----------------------------------------------------")
//            console.debug("Diary visible elements (index changed)", numVisibleComponents(diary))
//        }
//    }
}
