import QtQuick 1.1
import Common 1.0

// Collection of sheets
QtQuickItem {
    // Visible sheets are not null in sheets
    property ObjectContainer sheets: ObjectContainer {}
    // Some sheets can be omitted while turning
    property int omittedSheetsBegin
    property int omittedSheetsEnd
    // Num of created sheets for debug purpose
    property int _createdSheets

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

    id: sheetContainer
    anchors.fill: parent

    // Create a sheet for the specified index. Index must be >= 0.
    // If the sheet is visible (already created) it will be returned.
    // This function is for internal use.
    function createSheetAt(index, turn) {
        if (index < 0) return null

        var sheet = sheets.at(index)
        if (sheet) {
            return sheet
        }

        // Take from pool or create
        if (freeSheetsPool.children.length > 0) {
            sheet = freeSheetsPool.children[freeSheetsPool.children.length - 1]
            sheet.parent = sheetContainer
        } else {
            sheet = sheetComponent.createObject(sheetContainer)
            if (++_createdSheets > 15) {
                console.log("Too much active sheets in diary. Use fewer sheets in the turning.", _createdSheets)
            }
        }
        sheet.turn = turn
        sheet.sheetIndex = index
        sheets.setAt(index, sheet)

        // Set dirt and content
        var p = index * 2
        if (pageDirt && pageDirt.length) {
            var d = pageDirt[p % pageDirt.length]
            sheet.front.dirtSource = d[0]
            sheet.front.dirtScale = d[1]
            sheet.front.dirtRotation = d[2]
            sheet.front.dirtOffset = d[3]
            d = pageDirt[(p + 1) % pageDirt.length]
            sheet.back.dirtSource = d[0]
            sheet.back.dirtScale = d[1]
            sheet.back.dirtRotation = d[2]
            sheet.back.dirtOffset = d[3]
        }

        if (p < pageSources.count()) {
            pageSources.at(p).parent = sheet.front
        } else if (overPageSourceFront) {
            overPageSourceFront.parent = sheet.front
        }

        if (p + 1 < pageSources.count()) {
            pageSources.at(p + 1).parent = sheet.back
        } else if (overPageSourceBack) {
            overPageSourceBack.parent = sheet.back
        }

        Utils.asyncLoading.setAsync(sheet, true)
        Utils.setInactive(sheet, false)

        return sheet
    }

    // Frees sheets previously created with createSheetAt
    // This function is for internal use.
    function freeSheetAt(index) {
        var sheet = sheets.at(index)
        if (sheet) {
            sheet.turnAnimation.complete()

            Utils.setInactive(sheet, true)

            sheet.front.clear()
            sheet.back.clear()

            sheets.setAt(sheet.sheetIndex, null)
            sheet.sheetIndex = -1
            sheet.parent = freeSheetsPool
        }
    }

    function nextSheetIndex(sheetIndex) {
        return omittedSheetsBegin <= sheetIndex && sheetIndex < omittedSheetsEnd
                                    ? omittedSheetsEnd : sheetIndex + 1
    }

    function prevSheetIndex(sheetIndex) {
        return omittedSheetsBegin < sheetIndex && sheetIndex <= omittedSheetsEnd
                ? omittedSheetsBegin : sheetIndex - 1
    }

    Component.onCompleted: {
        createSheetAt(currentSheetIndex, 1)
        createSheetAt(currentSheetIndex - 1, -1)
    }

    onChildrenChanged: {
        if (children.length > 15) console.log("ERROR: Too much diary sheets visible simultaneously")
        // console.debug("sheetContainer.children.length", children.length)
    }

    function _updateSheets() {
        for (var i = sheets.count(); --i >= 0;) {
            freeSheetAt(i)
        }
        createSheetAt(currentSheetIndex, 1)
        createSheetAt(currentSheetIndex - 1, -1)
    }


    Connections {
        target: diary
        onCurrentSheetIndexChanged: {
            // For hidden diary do not turn sheets, just create the new ones for currentSheetIndex
            if (!diary.visible) {
                _updateSheets()
            }
        }
        onVisibleChanged: {
            // Restore diary sheets if we mixed them up with chaotic clicks
            if (!diary.visible) {
                _updateSheets()
            }
        }
    }

    Connections {
        target: pageSources
        onDataChanged: _updateSheets()
    }

    Item { // Contains unused hidden sheets
        id: freeSheetsPool
        visible: false
    }

    Component { // Generates new sheets
        id: sheetComponent
        Sheet {}
    }

    // Controls currentSheet by flick
    MouseArea {
        property int flickStartX
        property int flickStartY
        property int prevMouseX
        property int deltaMouseX

        anchors.fill: parent
        z: 1000

        onPressed: {
            // Do not turn a page out of the turnable range
            if (minTurnSheetIndex <= currentSheetIndex && currentSheetIndex <= maxTurnSheetIndex && !sheetTurningAnimation.running) {
                flickStartX = prevMouseX = mouseX
                flickStartY = mouseY
                pageDragging = true
            }
        }

        onPositionChanged: if (mouse.buttons === Qt.LeftButton && pageDragging) {
            if (flickStartX >= width / 2) {
                if (currentSheetIndex < maxTurnSheetIndex) {
                    var sheetRight = sheets.at(currentSheetIndex)
                    sheetRight.turnAnimation.complete()
                    var tr = 1 + 2 * (mouseX - flickStartX) / width
                    sheetRight.turn = Math.max(-1, Math.min(tr, 1))
                }
            } else {
                if (currentSheetIndex > minTurnSheetIndex) {
                    var sheetLeft = sheets.at(prevSheetIndex(currentSheetIndex))
                    sheetLeft.turnAnimation.complete()
                    var tl = -1 + 2 * (mouseX - flickStartX) / width
                    sheetLeft.turn = Math.max(-1, Math.min(tl, 1))
                }
            }
            deltaMouseX = mouseX - prevMouseX
            prevMouseX = mouseX
        }

        onReleased: if (pageDragging) {
            pageDragging = false
            var sheetRight = sheets.at(currentSheetIndex)
            var sheetLeft = sheets.at(prevSheetIndex(currentSheetIndex))

            if (flickStartX >= width / 2) {
                if (currentSheetIndex < maxTurnSheetIndex) {
                    if (sheetRight.turn < 0.8 && deltaMouseX <= 0) {
                       currentSheetIndex = nextSheetIndex(currentSheetIndex)
                    } else {
                        // Turn by click
                        if (mouseY > height * 0.85 && Math.abs(mouseX - flickStartX) < 10
                                && Math.abs(mouseY - flickStartY) < 10 && !sheetRight.turnAnimation.running) {
                            currentSheetIndex = nextSheetIndex(currentSheetIndex)
                        } else {
                            sheetRight.animateToCorrectPosition()

                            // Redirect click to the page content
                            if (Utils.isAncestorOf(sheetRight.front, objUnderCursor) && (!sheetLeft || sheetLeft.turn === -1)) {
                                objUnderCursor.clicked(mouse)
                            }
                        }
                    }
                }
            } else {
                if (currentSheetIndex > minTurnSheetIndex) {
                    if (sheetLeft.turn > -0.8 && deltaMouseX >= 0) {
                        currentSheetIndex = prevSheetIndex(currentSheetIndex)
                    } else {
                        // Turn by click
                        if (mouseY > height * 0.85 && Math.abs(mouseX - flickStartX) < 10
                                && Math.abs(mouseY - flickStartY) < 10 && !sheetLeft.turnAnimation.running) {
                            currentSheetIndex = prevSheetIndex(currentSheetIndex)
                        } else {
                            sheetLeft.animateToCorrectPosition()

                            // Redirect click to the page content
                            if (Utils.isAncestorOf(sheetLeft.back, objUnderCursor) && (!sheetRight || sheetRight.turn === 1)) {
                                objUnderCursor.clicked(mouse)
                            }
                        }
                    }
                }
            }
        }
    }
}
