import "Hint.js" as Hint
import QtQuick 1.1
import Common 1.0

// Base for Game component. Game is a container of other components: menu, locations, gui...
MouseArea {

    // Current active location. If null this is the menu mode.
    // This is a convenient property, readonly for the user.
    property Location activeLocation
    // Useful for save/load. Readonly.
    property int activeLocationIndex

    // Helps to detect whether object is within zoom zone
    property bool isInsideZoomZone

    // lamp is null by default. Can be overridden in Game.qml
    property QtObject lamp


    // Automatically filled by detecting Location as a child of Game
    property ObjectContainer locations: ObjectContainer {}

    // Apply object that is dragged by the user
    property ApplyObj draggingApplyObj

    // Currently zoomed zoom zone
    property ZoomZone zoomedZoomZone

    // Drag coordinates in game screen space
    property real screenDragX: activeLocation && activeLocation.activeScreen ?
                                   activeLocation.activeScreen.mapFromItem(root, dragX, dragY).x : 0
    property real screenDragY: activeLocation && activeLocation.activeScreen ?
                                   activeLocation.activeScreen.mapFromItem(root, dragX, dragY).y : 0

    property int _enumeratedObjectsGroupIndex: -1
    property int _ACTUAL_ENUMERATED_OBJECTS_GROUP_INDEX: -1 // For additional check

    // Fired when location changed
    signal locationChanged(variant location)
    // Fired when a game screen loading started
    signal screenLoadingStarted()
    // Fired when a game screen is loaded and visible
    signal screenLoaded(variant screen)
    // Fired when fading is comleted and screen images are loaded
    signal fadingCompleted(variant screen)
    // Fired when fade out is started and screen images are loaded
    signal fadeoutStarted(variant screen)
    // Fired when the game screen is left and hidden
    signal screenLeft(variant screen)
    // Fired when the game screen is destroyed after unloading
    signal screenUnloaded

    // Fired when apply object accepted
    signal applyObjAccepted(variant acceptor, variant applyObj)
    // Fired when apply object declined
    signal applyObjDeclined(variant decliner, variant applyObj)
    // Fired when apply object state changed
    signal applyObjStateChanged(variant applyObj)
    // Fired when ZoomZone state changed. BeltZoomZone is also taken into account
    signal zoomZoneStateChanged(variant zoomZone)
    // Fired when Conversation is opened or closed
    signal conversationVisibilityChanged(variant conversation)
    // Fired when ISpy found
    signal ispyFound(variant group, variant ispy)
    // Fired when ISpyGroup enabled/disabled
    signal ispyGroupEnabledChanged(variant group)
    // Fired when Obj clicked
    signal objClicked(variant obj)
    // Fired when hint shown
    signal hintShown(variant properties)
    // Fired when diary was opened
    signal diaryOpened(variant diary)
    // Fired when diary was closed
    signal diaryClosed(variant diary)

    // Fired when profile changed
    signal reset


    anchors.fill: parent
    drag.filterChildren: true
    objectName: "Game"


    onApplyObjStateChanged: {
        if (applyObj.state === "ReadyToApply") {
            draggingApplyObj = applyObj
        } else if (applyObj === draggingApplyObj) {
            draggingApplyObj = null
        }
    }

    onZoomZoneStateChanged: zoomedZoomZone = zoomZone.state === "Zoomed" ? zoomZone : null


    // Hides menu and all locations except the active one.
    function switchToLocation(location) {
        if (_enumeratedObjectsGroupIndex != _ACTUAL_ENUMERATED_OBJECTS_GROUP_INDEX) {
            console.log("ERROR: game object structure was changed between save-load")
            return
        }

        _makeChildrenInactive()

        // Set activeLocationIndex
        for (var i = 0; i < locations.count(); ++i) {
            if (locations.at(i) === location) {
                activeLocationIndex = i
                break
            }
        }
        activeLocation = location === menu ? null : location
        location.state = ""
        locationChanged(location)
        // Load belt pack and gui resources if not menu
        Utils.setInactive(beltPack, location === menu)
        Utils.setInactive(gui, location === menu)
    }

    // Hides all locations and shows menu.
    function switchToMenu() {
        switchToLocation(menu)
    }

    function _makeChildrenInactive() {
        // Reset to default state all children: menu, gui, all locations
        for (var i = 0; i < locations.count(); ++i) {
            locations.at(i).state = "Inactive"
        }
        beltPack.state = gui.state = menu.state = "Inactive"
    }

    // Finds a hint.
    // Returns dictionary {obj: o, applyObj: a, zoomZone: z}.
    // applyObj and zoomZone can be undefined if not applicable.
    // null can be returned if no hint found.
    // obj can be Jump (use toScreen !== undefined to check)
    function findHint() {
        if (!activeLocation || !activeLocation.activeScreen) {
            return null
        }

        // Set globals for Hint context
        Hint.beltPack = beltPack
        Hint.Utils = Utils

        var zzHint = Hint.findHintOnScreen(popupLayer)
        if (!zzHint) {
            popupLayer.closeAllWindows()
        }
        return zzHint
            || Hint.findHintOnScreen(activeLocation.activeScreen)
            || Hint.findHintInBeltZoomZones(beltPack.container)
            || Hint.findJump(activeLocation.screens[activeLocation.activeScreenIndex])
    }


    // Save/load support
    function saveState() {
        return {
            "activeLocationIndex" : activeLocationIndex,
            "_enumeratedObjectsGroupIndex": _enumeratedObjectsGroupIndex
        }
    }
    function loadState(data) {
        activeLocationIndex = data["activeLocationIndex"] || 0
        _enumeratedObjectsGroupIndex = data["_enumeratedObjectsGroupIndex"] || 0
    }

    Component.onCompleted: {
        // Fill locations
        for (var i = 0; i < children.length; ++i) {
            if (children[i].activeScreenIndex !== undefined) {
                locations.append(children[i])
            }
        }

        _enumeratedObjectsGroupIndex = Persistence.allocateEnumeratedObjectsGroup(objectName)
        _ACTUAL_ENUMERATED_OBJECTS_GROUP_INDEX = _enumeratedObjectsGroupIndex
    }


    Connections {
        target: root
        onIsApplicationFocusedChanged: {
            if (!isApplicationFocused && draggingApplyObj) {
                draggingApplyObj.returnAfterDragAndFireSignal()
            }
        }
    }

    // TODO: ScreenPreloader disabled because of inefficiency
//    ScreenPreloader {
//        id: screenPreloader
//    }
}
