import QtQuick 1.1
import Qt.labs.shaders 1.0
import Common 1.0

// Layer for pop-up windows, such as ZoomZoneContainer, Diary, message boxes.
// Should be placed within game component.
QtQuickItem {

    property alias effect: effect
    property alias effectSource: effectSource

    //effect.color does not work directly
    property alias effectColor: effect.color

    property QtQuickItem topWindow

    property bool isWindowOpen: children.length > 1 && children[1].scale === 1

    // Zoom effect for the layer blur texture
    property real zoomCenterX
    property real zoomCenterY
    property real zoomScale: 1

    property bool isTransient: true // Do not save/load state
    property bool isSkipForEnumeration: true // Do not enumerate on saving/loading

    id: self
    anchors.fill: parent

    onChildrenChanged: _adjustTopWindow()

    // Closes open windows
    function closeAllWindows() {
        for (var i = children.length; --i > 0;) {
            children[i].close()
        }
    }

    function _adjustTopWindow() {
        for (var i = children.length; --i > 0;) {
            var child = children[i]
            if (child.state !== "Closed" && !child.closed) {
                topWindow = child
                return
            }
        }
        topWindow = null
    }

    Behavior on zoomScale {
        NumberAnimation { easing.type: Easing.InOutQuad; duration: 400 }
    }

    Connections {
        ignoreUnknownSignals: true
        target: children.length > 1 ? children[children.length - 1] : null
        onStateChanged: _adjustTopWindow()
        onClosedChanged: _adjustTopWindow()
    }

    Connections {
        target: game
        onScreenLeft: zoomScale = 1
    }

    // Fast blur effect
    ShaderEffectItem {
        property color color: "#110"
        property variant pixelSize0: Qt.point(1.2 / width, 1.2 / height)
        property variant pixelSize1: Qt.point(-1.2 / width, 1.2 / height)
        property variant texCoordOffset: Qt.point(zoomCenterX * (1 - 1 / zoomScale) / width,
                                                  (height - zoomCenterY) * (1 - 1 / zoomScale) / height)
        property variant texCoordScale: Qt.point(1 / zoomScale, 1 / zoomScale)
        property variant source: effectSource

        id: effect
        anchors.fill: parent
        opacity: self.children.length > 1 && self.children[1].state !== "Closed" && !self.children[1].closed
        visible: opacity > 0
        blending: true

        ShaderEffectSource {
            id: effectSource
            sourceItem: activeLocation ? activeLocation.activeScreen : menu
            live: false; smooth: true; hideSource: effect.opacity == 1
            textureSize: Qt.size(effect.width, effect.height) // Reduce fbo recreation with screen zoom
        }

        Behavior on opacity {
            NumberAnimation { easing.type: Easing.InOutQuad; duration: 400 }
        }

        // Vertex shader is here to invert y for the flipped fbo texture. TODO: find a fix for the flipped fbo in C++
        vertexShader: "
            uniform highp mat4 qt_ModelViewProjectionMatrix;
            attribute highp vec4 qt_Vertex;
            attribute highp vec2 qt_MultiTexCoord0;
            uniform mediump vec2 texCoordOffset;
            uniform mediump vec2 texCoordScale;
            varying mediump vec2 qt_TexCoord0;
            void main(void)
            {
                qt_TexCoord0 = vec2(qt_MultiTexCoord0.x, 1. - qt_MultiTexCoord0.y) * texCoordScale + texCoordOffset;
                gl_Position = qt_ModelViewProjectionMatrix * qt_Vertex;
            }"

        fragmentShader: root.platformTraits & RootItem.SmallScreenPlatform ? simpleFragmentShader : complexFragmentShader

        property string simpleFragmentShader: "
            varying mediump vec2 qt_TexCoord0;
            uniform lowp sampler2D source;
            uniform lowp float qt_Opacity;
            uniform lowp vec4 color;
            void main(void)
            {
            lowp vec4 c = texture2D(source, qt_TexCoord0);
            gl_FragColor =  mix(c, color, 0.5) * qt_Opacity;
            }"

        property string complexFragmentShader: "
            varying mediump vec2 qt_TexCoord0;
            uniform lowp sampler2D source;
            uniform lowp float qt_Opacity;
            uniform lowp vec4 color;
            uniform mediump vec2 pixelSize0;
            uniform mediump vec2 pixelSize1;
            void main(void)
            {
            mediump vec4 c = texture2D(source, qt_TexCoord0 - pixelSize0);
                        c += texture2D(source, qt_TexCoord0 + pixelSize1);
                        c += texture2D(source, qt_TexCoord0 - pixelSize1);
                        c += texture2D(source, qt_TexCoord0 + pixelSize0);
            lowp float f = dot(c, vec4(0.33 * 0.25, 0.37 * 0.25, 0.3 * 0.25, 0));
            gl_FragColor =  mix(vec4(f, f, f, 1), color, 0.5) * qt_Opacity;
            }"

        // Block input. Obj type allows to overload cursor.
        ObjBase {
            property bool keepUnacceptedApplyObjDragged: true
            anchors.fill: parent
            hoverEnabled: true
            onPressed: if (isWindowOpen && topWindow.close !== undefined) topWindow.close()
        }
    }
}
