import QtQuick 1.1
import Qt.labs.shaders 1.0
import Common 1.0

// Helps to implement ropes in a game.
// Rope is a long vertical bitmap morphed by the control points.
// Actual length of the rope is determined by the control points.
ShaderEffectItem {
    property url source
    property Image image: Image { source: self.source; fillMode: Image.TileVertically }
    property real tile: 1
    property real tileOffset
    
    property real tileWidth: 8

    // Control points make a spline for the rope.
    property real x0
    property real y0
    property real x1
    property real y1
    property real x2
    property real y2
    property real x3
    property real y3
    property real x4
    property real y4

    // Variable width. Equal width by default.
    property real w0: tileWidth
    property real w1: tileWidth
    property real w2: tileWidth
    property real w3: tileWidth
    property real w4: tileWidth

    // Two parallel catmull-rom splines for the vertex shader
    property variant p0l: _leftPoint(x0, y0, x1 - x0, y1 - y0, w0)
    property variant p0r: _rightPoint(x0, y0, x1 - x0, y1 - y0, w0)
    property variant p1l: _leftPoint(x1, y1, x2 - x0, y2 - y0, w1)
    property variant p1r: _rightPoint(x1, y1, x2 - x0, y2 - y0, w1)
    property variant p2l: _leftPoint(x2, y2, x3 - x1, y3 - y1, w2)
    property variant p2r: _rightPoint(x2, y2, x3 - x1, y3 - y1, w2)
    property variant p3l: _leftPoint(x3, y3, x4 - x2, y4 - y2, w3)
    property variant p3r: _rightPoint(x3, y3, x4 - x2, y4 - y2, w3)
    property variant p4l: _leftPoint(x4, y4, x4 - x3, y4 - y3, w4)
    property variant p4r: _rightPoint(x4, y4, x4 - x3, y4 - y3, w4)

    property variant imgSampler: image

    id: self
    meshResolution: Qt.size(1, 50)
    width: 10
    height: 10
    shaderProgramName: "Rope" // Name to share program between ShaderEffectItem instances

    function _leftPoint(x, y, dx, dy, w) {
        var m = 0.5 * w / Math.sqrt(dx * dx + dy * dy)
        return Qt.point(x + dy * m, y - dx * m)
    }

    function _rightPoint(x, y, dx, dy, w) {
        var m = 0.5 * w / Math.sqrt(dx * dx + dy * dy)
        return Qt.point(x - dy * m, y + dx * m)
    }

    vertexShader: "
        uniform mediump vec2 p0l;
        uniform mediump vec2 p1l;
        uniform mediump vec2 p2l;
        uniform mediump vec2 p3l;
        uniform mediump vec2 p4l;
        uniform mediump vec2 p0r;
        uniform mediump vec2 p1r;
        uniform mediump vec2 p2r;
        uniform mediump vec2 p3r;
        uniform mediump vec2 p4r;
        uniform mediump float tile;
        uniform mediump float tileOffset;
        uniform mediump mat4 qt_ModelViewProjectionMatrix;
        attribute mediump vec4 qt_Vertex;
        attribute mediump vec2 qt_MultiTexCoord0;
        varying mediump vec2 qt_TexCoord0;

        mediump vec2 hermite(mediump float t, mediump vec2 p0, mediump vec2 p1, mediump vec2 p2, mediump vec2 p3)
        {
            return 0.5 * (2. * p1 + (-p0 + p2) * t + (2. * p0 - 5. * p1 + 4. * p2 - p3) * t * t
                                    + (-p0 + 3. * p1 - 3. * p2 + p3) * t * t * t);
        }

        void main(void)
        {
            qt_TexCoord0 = qt_MultiTexCoord0;
            qt_TexCoord0.y *= tile;
            qt_TexCoord0.y -= tileOffset;
            mediump vec2 v;
            mediump float y = qt_MultiTexCoord0.y;
            if (qt_MultiTexCoord0.x >= 0.5)
            {
                if (y < 0.5)
                {
                    if (y < 0.25)
                        v = hermite(y * 4., p0l - (p1l - p0l), p0l, p1l, p2l);
                    else
                        v = hermite(y * 4. - 1., p0l, p1l, p2l, p3l);
                }
                else
                {
                    if (y < 0.75)
                        v = hermite(y * 4. - 2., p1l, p2l, p3l, p4l);
                    else
                        v = hermite(y * 4. - 3., p2l, p3l, p4l, p4l + (p4l - p3l));
                }
            }
            else
            {
                if (y < 0.5)
                {
                    if (y < 0.25)
                        v = hermite(y * 4., p0r - (p1r - p0r), p0r, p1r, p2r);
                    else
                        v = hermite(y * 4. - 1., p0r, p1r, p2r, p3r);
                }
                else
                {
                    if (y < 0.75)
                        v = hermite(y * 4. - 2., p1r, p2r, p3r, p4r);
                    else
                        v = hermite(y * 4. - 3., p2r, p3r, p4r, p4r + (p4r - p3r));
                }
            }
            gl_Position = qt_ModelViewProjectionMatrix * vec4(v, 0, 1);
        }"

    fragmentShader: "
        varying mediump vec2 qt_TexCoord0;
        uniform lowp sampler2D imgSampler;
        uniform lowp float qt_Opacity;
        void main(void)
        {
            gl_FragColor =  texture2D(imgSampler, qt_TexCoord0) * qt_Opacity;
        }"
}
