import "Hint.js" as Hint
import QtQuick 1.1
import Common 1.0


// Represents screen interface for other screens and components.
// Loads/unloads screen content.
QtObject {
    // Screen qml url.
    property url source
    // Set automatically. null if the screen is not loaded.
    property QtQuickItem screen
    // Set automatically in Hint.js. Can be manually reset to false to be recalculated.
    property bool hintable

    // Set automatically
    property string _screenObjectName
    property int _enumeratedObjectsGroupIndex: -1
    property int _ACTUAL_ENUMERATED_OBJECTS_GROUP_INDEX: -1 // For additional check

    // Fired when the game screen loading started
    signal screenLoadingStarted()
    // Fired when the game screen is loaded and visible
    signal screenLoaded()
    // Fired when the game screen is left and hidden
    signal screenLeft()

    id: self

    function load() {
        if (_enumeratedObjectsGroupIndex != _ACTUAL_ENUMERATED_OBJECTS_GROUP_INDEX) {
            console.log("ERROR: screen loaders structure was changed between save-load")
            return
        }

        if (!screen) {
            screenLoadingStarted()
            game.screenLoadingStarted()
            _nextFrameLoadingTimer.start()
        }
    }

    function unload() {
        screen.screenLeft()
        game.screenLeft(screen)
        screenLeft()
        Utils.setInactive(screen, true) // Destroy is delayed, so make inactive and invisible
        destroyScreen(screen)
        screen = null
        game.screenUnloaded()
        Persistence.clearEnumeratedObjectsGroup(_enumeratedObjectsGroupIndex)
    }

    // The function is used by screen cache
    function loadScreenInContext(skipPersistence) {
        var component = Qt.createComponent(source)
        if (component) {
            var screen = component.createObject(screenContainer)
            if (!screen) {
                var estr = component.errorString()
                var firstSpaceIndex = estr.indexOf(" ")
                console.log(estr.substring(0, firstSpaceIndex) + ":" + estr.substring(firstSpaceIndex))
                return null
            }

            Utils.asyncLoading.setAsync(screen, true)

            screen.objectName = _screenObjectName

            if (!skipPersistence) {
                persistenceController.loadObject(screen.objectName, screen)
            }
            return screen
        }
    }

    // Destroys screen previously loaded by loadScreenInContext()
    function destroyScreen(screen) {
        //_setPointersCheckCount(10)
        _resetObjUnderCursor()
        screen.visible = false
        screen.destroy()

        _checkPointersTimer.restart()
    }

    onScreenLeft: {
        // Adjust hint properties for jump hinting on other screens
        Hint.beltPack = beltPack
        Hint.Utils = Utils
        hintable = Hint.findHintOnScreen(screen) !== null
    }

    Component.onCompleted: {
        // Set objectName for persistence
        var src = source.toString().split("/")
        src = src[src.length - 1]
        _screenObjectName = location.objectName + "/" + src.substring(0, src.length - 4)

        _enumeratedObjectsGroupIndex = Persistence.allocateEnumeratedObjectsGroup(_screenObjectName)
        _ACTUAL_ENUMERATED_OBJECTS_GROUP_INDEX = _enumeratedObjectsGroupIndex
    }

    property Timer _nextFrameLoadingTimer: Timer {
        interval: 1
        onTriggered: {
            screen = loadScreenInContext()
            if (screen) {
                Utils.setInactive(screen, false)
                screenLoaded()
                game.screenLoaded(screen)
                screen.screenLoaded()
            }
        }
    }

    property Timer _checkPointersTimer: Timer {
        interval: 10
        onTriggered: {
            Utils._checkPointersValidity(root)
        }
    }
}
