import QtQuick 1.1

// Object that has a hidden content and expands on click to show its content.
// By convention ZoomZone must have header and container children.
// Header is a visible part when ZoomZone is closed, container is a visible part when ZoomZone is opened (Zoomed).
// See ZoomZone in the BaseGame demo project as a template for your ZoomZone.
Obj {

    // If true the zoom zone is switched to "Empty" state when no visible children
    property bool autocloseIfEmpty : true

    // If you add a new state and this state is similar to "Zoomed" one
    // you should put your state in zoomStates.
    property variant zoomStates: ["Zoomed"]

    // Helps to detect whether object is within zoom zone
    property bool isInsideZoomZone: true

    property bool keepUnacceptedApplyObjDragged: true

    id: self
    width: header.width
    height: header.height
    state: "Closed"
    cursor: state === "Closed" ? "Container" : ""

    onStateChanged: {
        if (container) {
            if (zoomStates.indexOf(state) >= 0) {
                // Close other zoom zones and open container
                popupLayer.closeAllWindows()
                container.open()
            } else {
                container.close()
            }
        }

        // Fire signal for loaded screens' zoom zones and inventory zoom zones
        if (!Utils.isAncestorOf(activeLocation, self) || isLoaded) {
            game.zoomZoneStateChanged(self)
        }
    }

    onClicked: if (state === "Closed") state = "Zoomed"
    // Treat obj applying as a click
    onApplyObjAccepted: if (state === "Closed") state = "Zoomed"
    onApplyObjDeclined: if (state === "Closed") state = "Zoomed"

    // Container can be closed by command from popup layer
    Connections {
        target: container
        onStateChanged: {
            if (container.state === "Closed" && zoomStates.indexOf(state) >= 0) {
                state = "Closed"
            }
        }
    }

    // Set "Empty" state when no active objects in the container
    Connections {
        target: autocloseIfEmpty ? container.contentContainer : null
        onChildrenChanged: {
            // Count Obj in the zoom zone container
            if (_countActiveObj(container.contentContainer) === 0) {
                state = "Empty"
            }
        }
    }

    function _countActiveObj(component) {
        var count = 0
        for (var i = 0; i < component.children.length; ++i) {
            var child = component.children[i]
            if (child.pickingShape && child.enabled && child.state !== "Done" && child.state !== "Applied") {
                ++count
            }
            count += _countActiveObj(child)
        }
        return count
    }

    // Convenient method
    function closeImmediately() {
        state = "Closed"
    }

    // Convenient method
    function kill() {
        state = "Done"
    }

    // Close container with a delay. Very useful when the item taken from the container.
    function close(delay) {
        closeDelayPause.duration = delay || 500
        closeDelayAnimation.start()
    }

    // Close container with a delay and then put it in Done state. Very useful when the item taken from the container.
    function closeAndKill(delay) {
        killDelayPause.duration = delay || 500
        killDelayAnimation.start()
    }

    SequentialAnimation {
        id: closeDelayAnimation
        PauseAnimation { id: closeDelayPause }
        ScriptAction { script: state = "Closed" }
    }

    SequentialAnimation {
        id: killDelayAnimation
        PauseAnimation { id: killDelayPause }
        ScriptAction { script: { state = "Closed"; enabled = false } }
        PauseAnimation { duration: 200 }
        ScriptAction { script: state = "Done" }
    }
}
